import pandas as pd


def extract_date(dataframe, date_feature='datetime'):
    df_ = dataframe.copy()
    time_data = pd.DatetimeIndex(df_[date_feature])
    df_['date'] = time_data.date
    df_['day'] = time_data.day
    df_['month'] = time_data.month
    df_['year'] = time_data.year
    df_['hour'] = time_data.hour
    df_['day_of_week'] = time_data.dayofweek
    df_['week_of_year'] = time_data.weekofyear


def train_test_split_by_day(df, cutoff_day: int, day_feature='day'):
    train = df[df[day_feature] <= cutoff_day].reset_index(drop=True)
    test = df[df[day_feature] > cutoff_day].reset_index(drop=True)
    return train, test



import emoji
import re
import itertools

# A list of CONTRACTION_EN from http://stackoverflow.com/questions/19790188/expanding-english-language-contractions-in-python
CONTRACTION_EN = {
    "ain't": "am not",
    "aren't": "are not",
    "can't": "cannot",
    "can't've": "cannot have",
    "'cause": "because",
    "could've": "could have",
    "couldn't": "could not",
    "couldn't've": "could not have",
    "didn't": "did not",
    "doesn't": "does not",
    "don't": "do not",
    "hadn't": "had not",
    "hadn't've": "had not have",
    "hasn't": "has not",
    "haven't": "have not",
    "he'd": "he would",
    "he'd've": "he would have",
    "he'll": "he will",
    "he's": "he is",
    "how'd": "how did",
    "how'll": "how will",
    "how's": "how is",
    "i'd": "i would",
    "i'll": "i will",
    "i'm": "i am",
    "i've": "i have",
    "isn't": "is not",
    "it'd": "it would",
    "it'll": "it will",
    "it's": "it is",
    "let's": "let us",
    "ma'am": "madam",
    "mayn't": "may not",
    "might've": "might have",
    "mightn't": "might not",
    "must've": "must have",
    "mustn't": "must not",
    "needn't": "need not",
    "oughtn't": "ought not",
    "shan't": "shall not",
    "sha'n't": "shall not",
    "she'd": "she would",
    "she'll": "she will",
    "she's": "she is",
    "should've": "should have",
    "shouldn't": "should not",
    "that'd": "that would",
    "that's": "that is",
    "there'd": "there had",
    "there's": "there is",
    "they'd": "they would",
    "they'll": "they will",
    "they're": "they are",
    "they've": "they have",
    "wasn't": "was not",
    "we'd": "we would",
    "we'll": "we will",
    "we're": "we are",
    "we've": "we have",
    "weren't": "were not",
    "what'll": "what will",
    "what're": "what are",
    "what's": "what is",
    "what've": "what have",
    "where'd": "where did",
    "where's": "where is",
    "who'll": "who will",
    "who's": "who is",
    "won't": "will not",
    "wouldn't": "would not",
    "you'd": "you would",
    "you'll": "you will",
    "you're": "you are",
    "thx": "thanks",
    "lool": "lol"
}

# Thanks to https://www.kaggle.com/rftexas/text-only-kfold-bert
ABBREVATION_EN = {
    "$": " dollar ",
    "€": " euro ",
    "4ao": "for adults only",
    "a.m": "before midday",
    "a3": "anytime anywhere anyplace",
    "aamof": "as a matter of fact",
    "acct": "account",
    "adih": "another day in hell",
    "afaic": "as far as i am concerned",
    "afaict": "as far as i can tell",
    "afaik": "as far as i know",
    "afair": "as far as i remember",
    "afk": "away from keyboard",
    "app": "application",
    "approx": "approximately",
    "apps": "applications",
    "asap": "as soon as possible",
    "asl": "age, sex, location",
    "atk": "at the keyboard",
    "ave.": "avenue",
    "aymm": "are you my mother",
    "ayor": "at your own risk",
    "b&b": "bed and breakfast",
    "b+b": "bed and breakfast",
    "b.c": "before christ",
    "b2b": "business to business",
    "b2c": "business to customer",
    "b4": "before",
    "b4n": "bye for now",
    "b@u": "back at you",
    "bae": "before anyone else",
    "bak": "back at keyboard",
    "bbbg": "bye bye be good",
    "bbc": "british broadcasting corporation",
    "bbias": "be back in a second",
    "bbl": "be back later",
    "bbs": "be back soon",
    "be4": "before",
    "bfn": "bye for now",
    "blvd": "boulevard",
    "bout": "about",
    "brb": "be right back",
    "bros": "brothers",
    "brt": "be right there",
    "bsaaw": "big smile and a wink",
    "btw": "by the way",
    "bwl": "bursting with laughter",
    "c/o": "care of",
    "cet": "central european time",
    "cf": "compare",
    "cia": "central intelligence agency",
    "csl": "can not stop laughing",
    "cu": "see you",
    "cul8r": "see you later",
    "cv": "curriculum vitae",
    "cwot": "complete waste of time",
    "cya": "see you",
    "cyt": "see you tomorrow",
    "dae": "does anyone else",
    "dbmib": "do not bother me i am busy",
    "diy": "do it yourself",
    "dm": "direct message",
    "dwh": "during work hours",
    "e123": "easy as one two three",
    "eet": "eastern european time",
    "eg": "example",
    "embm": "early morning business meeting",
    "encl": "enclosed",
    "encl.": "enclosed",
    "etc": "and so on",
    "faq": "frequently asked questions",
    "fawc": "for anyone who cares",
    "fb": "facebook",
    "fc": "fingers crossed",
    "fig": "figure",
    "fimh": "forever in my heart",
    "ft.": "feet",
    "ft": "featuring",
    "ftl": "for the loss",
    "ftw": "for the win",
    "fwiw": "for what it is worth",
    "fyi": "for your information",
    "g9": "genius",
    "gahoy": "get a hold of yourself",
    "gal": "get a life",
    "gcse": "general certificate of secondary education",
    "gfn": "gone for now",
    "gg": "good game",
    "gl": "good luck",
    "glhf": "good luck have fun",
    "gmt": "greenwich mean time",
    "gmta": "great minds think alike",
    "gn": "good night",
    "g.o.a.t": "greatest of all time",
    "goat": "greatest of all time",
    "goi": "get over it",
    "gps": "global positioning system",
    "gr8": "great",
    "gratz": "congratulations",
    "gyal": "girl",
    "h&c": "hot and cold",
    "hp": "horsepower",
    "hr": "hour",
    "hrh": "his royal highness",
    "ht": "height",
    "ibrb": "i will be right back",
    "ic": "i see",
    "icq": "i seek you",
    "icymi": "in case you missed it",
    "idc": "i do not care",
    "idgadf": "i do not give a damn fuck",
    "idgaf": "i do not give a fuck",
    "idk": "i do not know",
    "ie": "that is",
    "i.e": "that is",
    "ifyp": "i feel your pain",
    "IG": "instagram",
    "iirc": "if i remember correctly",
    "ilu": "i love you",
    "ily": "i love you",
    "imho": "in my humble opinion",
    "imo": "in my opinion",
    "imu": "i miss you",
    "iow": "in other words",
    "irl": "in real life",
    "j4f": "just for fun",
    "jic": "just in case",
    "jk": "just kidding",
    "jsyk": "just so you know",
    "l8r": "later",
    "lb": "pound",
    "lbs": "pounds",
    "ldr": "long distance relationship",
    "lmao": "laugh my ass off",
    "lmfao": "laugh my fucking ass off",
    "lol": "laughing out loud",
    "ltd": "limited",
    "ltns": "long time no see",
    "m8": "mate",
    "mf": "motherfucker",
    "mfs": "motherfuckers",
    "mfw": "my face when",
    "mofo": "motherfucker",
    "mph": "miles per hour",
    "mr": "mister",
    "mrw": "my reaction when",
    "ms": "miss",
    "mte": "my thoughts exactly",
    "nagi": "not a good idea",
    "nbc": "national broadcasting company",
    "nbd": "not big deal",
    "nfs": "not for sale",
    "ngl": "not going to lie",
    "nhs": "national health service",
    "nrn": "no reply necessary",
    "nsfl": "not safe for life",
    "nsfw": "not safe for work",
    "nth": "nice to have",
    "nvr": "never",
    "nyc": "new york city",
    "oc": "original content",
    "og": "original",
    "ohp": "overhead projector",
    "oic": "oh i see",
    "omdb": "over my dead body",
    "omg": "oh my god",
    "omw": "on my way",
    "p.a": "per annum",
    "p.m": "after midday",
    "pm": "prime minister",
    "poc": "people of color",
    "pov": "point of view",
    "pp": "pages",
    "ppl": "people",
    "prw": "parents are watching",
    "ps": "postscript",
    "pt": "point",
    "ptb": "please text back",
    "pto": "please turn over",
    "qpsa": "what happens",  # "que pasa",
    "ratchet": "rude",
    "rbtl": "read between the lines",
    "rlrt": "real life retweet",
    "rofl": "rolling on the floor laughing",
    "roflol": "rolling on the floor laughing out loud",
    "rotflmao": "rolling on the floor laughing my ass off",
    "rt": "retweet",
    "ruok": "are you ok",
    "sfw": "safe for work",
    "sk8": "skate",
    "smh": "shake my head",
    "sq": "square",
    "srsly": "seriously",
    "ssdd": "same stuff different day",
    "tbh": "to be honest",
    "tbs": "tablespooful",
    "tbsp": "tablespooful",
    "tfw": "that feeling when",
    "thks": "thank you",
    "tho": "though",
    "thx": "thank you",
    "tia": "thanks in advance",
    "til": "today i learned",
    "tl;dr": "too long i did not read",
    "tldr": "too long i did not read",
    "tmb": "tweet me back",
    "tntl": "trying not to laugh",
    "ttyl": "talk to you later",
    "u": "you",
    "u2": "you too",
    "u4e": "yours for ever",
    "utc": "coordinated universal time",
    "w/": "with",
    "w/o": "without",
    "w8": "wait",
    "wassup": "what is up",
    "wb": "welcome back",
    "wtf": "what the fuck",
    "wtg": "way to go",
    "wtpa": "where the party at",
    "wuf": "where are you from",
    "wuzup": "what is up",
    "wywh": "wish you were here",
    "yd": "yard",
    "ygtr": "you got that right",
    "ynk": "you never know",
    "zzz": "sleeping bored and tired"
}


def remove_CONTRACTION(text):
    return CONTRACTION_EN[text.lower()] if text.lower() in CONTRACTION_EN.keys() else text


def remove_abbrevation_ENABBREVATION_EN(text):
    return ABBREVATION_EN[text.lower()] if text.lower() in ABBREVATION_EN.keys() else text


def to_lowercase(text):
    text = text.lower()
    return text


def remove_ponctuation(text):
    # Remove punctuation
    text = ' '.join(re.sub("[\.\,\!\?\:\;\-\=\/\|\'\(\']", " ", text).split())
    return text


def remove_emoji(text):
    # Remove emoji
    text = emoji.demojize(text)
    text = text.replace(":", " ")
    text = ' '.join(text.split())
    text = re.sub("([^\x00-\x7F])+", " ", text)
    return text


def remove_hashtag(text):
    text = re.sub(r'#', '', text)  # Remove hashtag
    return text


def remove_line_breaks(text):
    text = re.sub(r'\n', ' ', text)  # Remove line breaks
    return text


def remove_extraspace_leading_trailing(text):
    # Remove leading, trailing, and extra spaces
    text = re.sub('\s+', ' ', text).strip()
    return text


def remove_links(text):
    text = re.sub(r'https?://\S+', '', text)  # Remove link
    return text


def remove_mentions(text):
    text = re.sub(r'@\w+', '', text)  # Remove mentions
    return text


def find_hashtags(tweet):
    return " ".join([match.group(0)[1:] for match in re.finditer(r"#\w+", tweet)]) or 'no'


def find_mentions(tweet):
    return " ".join([match.group(0)[1:] for match in re.finditer(r"@\w+", tweet)]) or 'no'


def find_links(tweet):
    return " ".join([match.group(0)[:] for match in re.finditer(r"https?://\S+", tweet)]) or 'no'



import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
import minisom
from sklearn.decomposition import PCA


def pca_visualisation(X, y, colordict, labeldict, markerdict, alphadict, figsize=(10, 10)):
    '''
    plot a visualisation of the data using the two main composant of a PCA. 

    Parameters
    ----------

    X : the explaining features, they should be normalised (numpy)

    y : the target feature (numpy)

    colordict : the color of each class present in y

    labeldict : the label of each class present in y

    markerdict : the marker of each class present in y

    alphadict :the alpha of each class present in y

    figsize : the size of the figure

    Return
    -------

    explained_variance_ratio : the ratio of the explained variance obtained with the PCA.

    Examples
    --------
    >>> from izza import pca_visualisation
    >>> cdict = {0 : "green", 1 : "red"}
    >>> labl  = {0: "healthy", 1 : "sick"}
    >>> marker= {0 : "o", 1: "*"}
    >>> alpha = {0: 0.5, 1: 0.5} 
    >>> pca_visualisation(X,y,cdict,labl,marker,alpha)
    '''

    pca = PCA(n_components=2)
    X_pca = pca.fit_transform(X)

    cdict = colordict
    labl = labeldict
    marker = markerdict
    alpha = alphadict

    _ = plt.figure(figsize=figsize)

    for l in np.unique(y):
        ix = np.where(y == l)
        plt.scatter(X_pca[ix, 0], X_pca[ix, 1], c=cdict[l],
                    s=40, label=labl[l], marker=marker[l], alpha=alpha[l])

    plt.xlabel("first principal component")
    plt.ylabel("second principal component")
    plt.legend()

    return pca.explained_variance_ratio_


def missingData(data):
    """Return a pandas dataframe of the missing observations by variables (number and percentage) and plot 
    a graph of those missing values by variable.
    """
    total = data.isnull().sum().sort_values(ascending=False)
    percent = (data.isnull().sum()/data.isnull().count()
               * 100).sort_values(ascending=False)
    md = pd.concat([total, percent], axis=1, keys=['Total', 'Percent'])
    md = md[md["Percent"] > 0]
    sns.set(style='darkgrid')
    plt.figure(figsize=(8, 4))
    plt.xticks(rotation='90')
    sns.barplot(md.index, md["Percent"], color="g", alpha=0.8)
    plt.xlabel('Features', fontsize=15)
    plt.ylabel('Percent of missing values', fontsize=15)
    plt.title('Percent missing data by feature', fontsize=15)
    return md


def tabcategorial(data, variable, hue):
    "return a pandas dataframe containing data from the category by class (useful for binary classification)"
    pass


def camembert_plot(dataset, variable, target, cutoff, plot=True, figsize=(8, 8)):
    '''
    plot a camembert plot for a variable in the case of the non fraudster
    population and an other one for the fraudster population and print a
    table containing the data used to create the figures.

    Parameters
    ----------

    dataset : a pandas dataframe

    variable : the name of the variable to plot

    target : the name of the target variable to take into account

    cutoff : the minimum importance for a modality

    plot : a boolean representing if it is needed to plot the data

    figsize : the size of the figure

    Return
    -------

    table : the table containing the data used to create the figures

    '''
    data_nofrd = dataset[dataset[target] == 0]
    data_fraud = dataset[dataset[target] == 1]

    nofraudster = (data_nofrd[variable].value_counts() / len(data_nofrd)) * 100
    fraudsters = (data_fraud[variable].value_counts() / len(data_fraud)) * 100

    nofraudster = nofraudster.to_frame()
    fraudsters = fraudsters.to_frame()

    if plot:
        nofraudster[nofraudster[variable] > cutoff].plot.pie(
            variable, figsize=(5, 5))
        fraudsters[fraudsters[variable] > cutoff].plot.pie(
            variable, figsize=(5, 5))

    result = pd.concat([nofraudster, fraudsters], axis=1, join='inner')
    result.columns = ['not fraudsters', 'fraudsters']

    return result


def kohohen_maps(X, y, som, size_x, size_y):
    '''
    plot a kohohen maps, the number in red is the proportion of fraudster in
    each cell and the color represent the distance of each withe the
    neighbouring cells.

    Parameters
    ----------

    X : a numpy array with explaining features

    y : a numpy array with the target feature

    som : a minisom instance

    size_x : the size of the grid (width)

    size_y : the size of the grid (height)

    '''

    plt.figure(figsize=(size_x, size_y))

    plt.pcolor(som.distance_map().T, cmap='bone_r')
    plt.colorbar()

    fraudsters = {}
    total = {}
    peri = {}

    for cnt, xx in enumerate(X):
        w = som.winner(xx)  # getting the winner

        if str(w[0] + .3) + str(w[1] + .3) in fraudsters:

            calcul1 = fraudsters[str(w[0] + .3) + str(w[1] + .3)]
            calcul2 = total[str(w[0] + .3) + str(w[1] + .3)]

            fraudsters[str(w[0] + .3) + str(w[1] + .3)] = calcul1 + y[cnt]
            total[str(w[0] + .3) + str(w[1] + .3)] = calcul2 + 1

        else:
            fraudsters[str(w[0] + .3) + str(w[1] + .3)] = y[cnt]
            total[str(w[0] + .3) + str(w[1] + .3)] = 1
            peri[str(w[0] + .3) + str(w[1] + .3)] = (w[0] + .3, w[1] + .3)

    proportion = {}

    for i in total:
        proportion[i] = round(fraudsters[i] / total[i], 3)

    for i in total:
        prop = proportion[i]
        lat, long = peri[i]

        plt.text(lat, long, s=prop, color="red")


def activation_frequencies(X, som, size_x, size_y):
    '''
    plot a kohohen maps, the color in each cell represent how many observation
    end up in the cell

    Parameters
    ----------

    X : a numpy array with explaining features

    som : a minisom instance

    size_x : the size of the grid (width)

    size_y : the size of the grid (height)

    '''
    plt.figure(figsize=(size_x, size_y))
    frequencies = som.activation_response(X)
    plt.pcolor(frequencies.T, cmap='Blues')
    plt.colorbar()
    plt.show()



from sklearn.metrics.cluster import contingency_matrix


def f_macro_score(label_observed, label_pred, cut_CP, cut_CR):
    '''
    assess the f-macro score

    Parameters
    ----------

    label_observed : the true class of the observations
        numpy array

    label_pred : the class predicted by the clustering algorithm
        numpy array

    cut_P : the minimum percentage of precision by cluster
        float

    cut_R : the minimum percentage of recall by cluster
        float

    Return
    -------

    f-macro:  the f-macro score

    '''
    # calculate contengency matrix for clustering

    matrix_contengency = contingency_matrix(label_observed, label_pred)

    # calculate precsion and recall for each cluster

    nb_cluster = 0
    classes = []

    for i in range(0, len(label_pred)):
        if label_pred[i] in classes:
            pass
        else:
            classes.append(label_pred[i])
            nb_cluster = nb_cluster + 1

    Plist = []
    Rlist = []

    for i in range(nb_cluster):
        P = matrix_contengency[1, i] / \
            matrix_contengency.sum(axis=0, dtype='float')[i]
        Plist.append(P)

    for j in range(nb_cluster):
        R = matrix_contengency[1, j] / \
            matrix_contengency.sum(axis=1, dtype='float')[1]
        Rlist.append(R)

    # select only viable cluster

    viable_cluster = []
    for i in range(0, nb_cluster):
        if Plist[i] >= cut_CP and Rlist[i] >= cut_CR:
            viable_cluster.append({'P': Plist[i],
                                   'R': Rlist[i]})

    # calculate global precision and global recall

    if len(viable_cluster) == 0:
        return 0
    else:
        global_precision = float(
            sum(cluster['P'] for cluster in viable_cluster)) / len(viable_cluster)
        global_recall = sum(cluster['R'] for cluster in viable_cluster)

        # calculate global f_score

        global_f_score = 2 * (global_precision * global_recall) / \
            (global_precision + global_recall)

        return global_f_score


def precision_macro_score(label_observed, label_pred, cut_CP, cut_CR):
    '''
    evaluate the mean precision on the cluster respecting the constraint

    Parameters
    ----------

    label_observed : the true class of the observations
        numpy array

    label_pred : the class predicted by the clustering algorithm
        numpy array

    cut_P : the minimum percentage of precision by cluster
        float

    cut_R : the minimum percentage of recall by cluster
        float

    Return
    -------

    f-precision: the mean precision on the cluster respecting the constraint

    '''
    # calculate contengency matrix for clustering

    matrix_contengency = contingency_matrix(label_observed, label_pred)

    # calculate precsion and recall for each cluster

    nb_cluster = 0
    classes = []

    for i in range(0, len(label_pred)):
        if label_pred[i] in classes:
            pass
        else:
            classes.append(label_pred[i])
            nb_cluster = nb_cluster + 1

    Plist = []
    Rlist = []

    for i in range(nb_cluster):
        P = matrix_contengency[1, i] / \
            matrix_contengency.sum(axis=0, dtype='float')[i]
        Plist.append(P)

    for j in range(nb_cluster):
        R = matrix_contengency[1, j] / \
            matrix_contengency.sum(axis=1, dtype='float')[1]
        Rlist.append(R)

    # select only viable cluster

    viable_cluster = []
    for i in range(0, nb_cluster):
        if Plist[i] >= cut_CP and Rlist[i] >= cut_CR:
            viable_cluster.append({'P': Plist[i],
                                   'R': Rlist[i]})

    # calculate global precision

    if len(viable_cluster) == 0:
        return 0
    else:
        global_precision = float(
            sum(cluster['P'] for cluster in viable_cluster)) / len(viable_cluster)

        return global_precision


def viable_clusters(label_observed, label_pred, cut_CP, cut_CR):
    '''
    evaluate the cluster respecting the constraints

    Parameters
    ----------

    label_observed : the true class of the observations
        numpy array

    label_pred : the class predicted by the clustering algorithm
        numpy array

    cut_P : the minimum percentage of precision by cluster
        float

    cut_R : the minimum percentage of recall by cluster
        float

    Return
    -------

    clusters: the clusters respecting the constraints

    '''

    matrix_contengency = contingency_matrix(label_observed, label_pred)

    # calculate precsion and recall for each cluster

    nb_cluster = 0
    classes = []

    for i in range(0, len(label_pred)):
        if label_pred[i] in classes:
            pass
        else:
            classes.append(label_pred[i])
            nb_cluster = nb_cluster + 1

    Plist = []
    Rlist = []

    for i in range(nb_cluster):
        P = matrix_contengency[1, i] / \
            matrix_contengency.sum(axis=0, dtype='float')[i]
        Plist.append(P)

    for j in range(nb_cluster):
        R = matrix_contengency[1, j] / \
            matrix_contengency.sum(axis=1, dtype='float')[1]
        Rlist.append(R)

    # select only viable cluster

    viable_cluster = []
    for i in range(0, nb_cluster):
        if Plist[i] >= cut_CP and Rlist[i] >= cut_CR:
            viable_cluster.append(i)

    return viable_cluster


def recall_macro_score(label_observed, label_pred, cut_CP, cut_CR):
    '''
    evaluate the sum of the recall on the cluster respecting the constraints

    Parameters
    ----------

    label_observed : the true class of the observations
        numpy array

    label_pred : the class predicted by the clustering algorithm
        numpy array

    cut_P : the minimum percentage of precision by cluster
        float

    cut_R : the minimum percentage of recall by cluster
        float

    Return
    -------

    f-recall: the sum of the recall on the cluster respecting the constraints

    '''
    # calculate contengency matrix for clustering

    matrix_contengency = contingency_matrix(label_observed, label_pred)

    # calculate precsion and recall for each cluster

    nb_cluster = 0
    classes = []

    for i in range(0, len(label_pred)):
        if label_pred[i] in classes:
            pass
        else:
            classes.append(label_pred[i])
            nb_cluster = nb_cluster + 1

    Plist = []
    Rlist = []

    for i in range(nb_cluster):
        P = matrix_contengency[1, i] / \
            matrix_contengency.sum(axis=0, dtype='float')[i]
        Plist.append(P)

    for j in range(nb_cluster):
        R = matrix_contengency[1, j] / \
            matrix_contengency.sum(axis=1, dtype='float')[1]
        Rlist.append(R)

    viable_cluster = []
    for i in range(0, nb_cluster):
        if Plist[i] >= cut_CP and Rlist[i] >= cut_CR:
            viable_cluster.append({'P': Plist[i],
                                   'R': Rlist[i]})

    # calculate global recall

    if len(viable_cluster) == 0:
        return 0
    else:
        global_recall = sum(cluster['R'] for cluster in viable_cluster)

        # calculate global recall

        return global_recall



import numpy as np
from sklearn.cluster import KMeans
from sklearn.base import BaseEstimator, clone
from sklearn.linear_model import LinearRegression


class ClusterBag(BaseEstimator):

    def __init__(self, estimatorcluster=None, estimatormodel=None):
        '''
        parameter : the estimator to use in the clustering part 
        the model to use in the supervised fit 
        parameter for both of them 
        '''

        if estimatorcluster == None:
            self.estimatorcluster = KMeans(n_clusters=3)
        else:
            self.estimatorcluster = estimatorcluster

        if estimatormodel == None:
            self.estimatormodel = LinearRegression()
        else:
            self.estimatormodel = estimatormodel

        self.estimators_ = {}

        self.is_fitted_cluster = 0
        self.is_fitted = 0

    def fit(self, X, y):
        '''
        fit the cluster model then the estimators models
        '''

        self.fit_estimator_cluster(X)
        self.fit_estimators(X, y, self.estimatormodel)

        self.is_fitted_cluster = 1

        return self

    def fit_estimator_cluster(self, X, y=None):
        '''
        Fit the cluster model
        '''

        # Fit the cluster estimator :

        self.labels_ = self.estimatorcluster.fit_predict(X)
        self.estimators_ = {}

        self.is_fitted_cluster = 1

    def fit_estimator(self, X, y, label_cluster, estimatorprediction):
        '''
        fit an estimator on the dataset related to his cluster
        '''

        # check if estimatorcluster is fitted

        if self.is_fitted_cluster == 0:
            raise Exception('You need to first fit the cluster estimator ')

        # Create a clone of our model

        estimator = clone(estimatorprediction)

        # Range our model cleanly

        ind = []
        ind = np.where(self.labels_ == label_cluster)[0]
        self.estimators_[label_cluster] = estimator

        # Fit our model

        self.estimators_[label_cluster].fit(X[ind], y[ind])

    def fit_estimators(self, X, y, estimatorprediction):
        '''
        fit all models for all clusters 
        '''

        # check if estimatorcluster is fitted

        if self.is_fitted_cluster == 0:
            raise Exception('You need to first fit the cluster estimator ')

        # Fit each cluster with our model

        for label in np.unique(self.labels_):
            self.fit_estimator(X, y, label, estimatorprediction)

        self.is_fitted = 1

    def predict(self, X):
        '''
        make a prediction by first finding the cluster and the model related to X 
        and then applying predict on the correct model
        '''

        # Check everything is fitted

        if self.is_fitted == 0:
            raise Exception('You need to first fit the cluster bag estimator')

        label = self.estimatorcluster.predict(X)

        prediction = []
        for i in range(0, len(label)):
            result = self.estimators_[label[i]].predict(X[i].reshape(1, -1))
            prediction.append(result)

        return np.array(prediction)

    def get_params(self, deep=True):
        out = {}

        out["estimatorcluster"] = self.estimatorcluster
        out["estimatormodel"] = self.estimatorcluster

        for model in self.estimators_:
            out["estimator"+str(model)] = self.estimators_[model]

        for key in self.estimatorcluster.get_params():
            out["estimatorcluster__" +
                str(key)] = self.estimatorcluster.get_params()[key]

        for key in self.estimatormodel.get_params():
            out["estimatormodel__" +
                str(key)] = self.estimatormodel.get_params()[key]

        for model in self.estimators_:
            for key in self.estimators_[model].get_params():
                out["estimator"+str(model)+"__"+str(key)
                    ] = self.estimators_[model].get_params()[key]

        return out



import numpy as np
from sklearn.metrics import recall_score, precision_score


def fun_precision(pourcentage, predicted_proba, X, y):
    '''
    evaluate the precision on the dataset X

    Parameters
    ----------

    pourcentage : the percentage of the population to take into account when
        calculating the precision and the recall

    predicted_proba : the probability of being a fraudster for each observation

    X : the target variable
        numpy array

    y : the explaining features
        numpy array

    Return
    -------

    precision_score : the precision on the dataset

    '''

    ratio = -pourcentage * int(X.shape[0] / 100)
    ind = np.argpartition(predicted_proba, ratio)[ratio:]
    predicted = np.zeros(y.shape[0])
    predicted[ind] = 1
    return precision_score(y_true=y, y_pred=predicted)


def fun_recall(pourcentage, predicted_proba, X, y):
    '''
    evaluate the recall on the dataset X

    Parameters
    ----------

    pourcentage : the percentage of the population to take into account
        when calculating the precision and the recall

    predicted_proba : the probability of being a fraudster
        for each observation

    X : the target variable
        numpy array

    y : the explaining features
        numpy array

    Return
    -------

    recall_score : the recall on the dataset

    '''

    ratio = -pourcentage * int(X.shape[0] / 100)
    ind = np.argpartition(predicted_proba, ratio)[ratio:]
    predicted = np.zeros(y.shape[0])
    predicted[ind] = 1
    return recall_score(y_true=y, y_pred=predicted)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['izza',
 'izza.exploration',
 'izza.modeling',
 'izza.modeling.feature_eng',
 'izza.validation']

package_data = \
{'': ['*']}

install_requires = \
['MiniSom>=2.2.8,<3.0.0',
 'matplotlib>=3.4.1,<4.0.0',
 'pandas>=1.2.3,<2.0.0',
 'scikit-learn>=0.24.1,<0.25.0']

setup_kwargs = {
    'name': 'izza',
    'version': '0.3.0',
    'description': 'my machine learning toolkit',
    'long_description': '# Izza \n\nIzza is my personal data science and machine learning toolbox.\n\nThere is mainly personal function for data visualisation: \n\n| function name | presentation |\n|--|--|\n| pca_visualisation| a pca visualtion (2 components) |\n| missingData| a tables showing missing data by features |\n| camembert_plot| a camembert plot |\n| kohohen maps|a kohohen maps with percentage of target class by neuron|\n| activation_frequencies| activation frequencies for a kohohen map |\n\nThere is methods to do model evaluation : \n\n\n| function name | presentation |\n|--|--|\n| fun_precision | precision at a determined percentage using predicted probabilities |\n| fun_recall| recall at a determined percentage using predicted probabilities |\n| f_macro_score | scorer allowing to find whether some clusters contain enough precision and recall for the target class|\n| precision_macro_score| scorer allowing to find whether some clusters contain enough precision for the target class |\n| recall_macro_score | scorer allowing to find whether some clusters contain enough recall for the target class|\n| viable_clusters| function allowing to know the interessing clusters found using the f_macro score.|\n\n\n\n',
    'author': 'Ismael Lachheb',
    'author_email': 'ismael.lachheb@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
